@@ -225,8 +225,8 @@ const enum Rec {
   // Once a stack reaches this depth (in .stack.length) force-reduce
   // it back to CutTo to avoid creating trees that overflow the stack
   // on recursive traversal.
-  CutDepth = 5000 * 3,
-  CutTo = 3000 * 3,
+  CutDepth = 2800 * 3,
+  CutTo = 2000 * 3,
   MaxLeftAssociativeReductionCount = 300,
   // The maximum number of non-recovering stacks to explore (to avoid
   // getting bogged down with exponentially multiplying stacks in
@@ -425,7 +425,8 @@ export class Parse implements PartialParse {
       let action = actions[i++], term = actions[i++], end = actions[i++]
       let last = i == actions.length || !split
       let localStack = last ? stack : stack.split()
-      localStack.apply(action, term, end)
+      let main = this.tokens.mainToken
+      localStack.apply(action, term, main ? main.start : localStack.pos, end)
       if (verbose)
         console.log(base + this.stackID(localStack) + ` (via ${(action & Action.ReduceFlag) == 0 ? "shift"
                      : `reduce of ${parser.getName(action & Action.ValueMask)}`} for ${
@@ -724,7 +725,7 @@ export class LRParser extends Parser {
       let prop = propSpec[0]
       if (typeof prop == "string") prop = (NodeProp as unknown as {[name: string]: NodeProp<any>})[prop]
       for (let i = 1; i < propSpec.length;) {
-        let next = propSpec[i++]
+        let next = propSpec[i++] as number
         if (next >= 0) {
           setProp(next as number, prop, propSpec[i++] as string)
         } else {
